package com.arms.api.util.alm;

import com.arms.api.issue.almapi.model.vo.지라이슈_데이터;
import com.arms.api.issue.almapi.model.dto.지라이슈조회_데이터;
import com.arms.api.issue.almapi.model.dto.CloudJiraIssueCreationFieldMetadata;
import com.arms.api.util.env.ApplicationProfile;
import com.arms.api.util.errors.ErrorLogUtil;
import com.arms.api.util.response.CommonResponse;
import com.arms.config.DevDurationConfig;
import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.internal.async.AsynchronousJiraRestClientFactory;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.AllArgsConstructor;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.core.env.Environment;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientResponseException;

import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.util.*;
import java.util.stream.Collectors;

import static com.arms.api.util.env.ApplicationProfile.DEV;
import static com.arms.config.ApplicationContextProvider.getBean;

@Slf4j
@Component
@AllArgsConstructor
public class 지라유틸 {

    private final 지라API_정보 지라API_정보;

    public WebClient 클라우드_통신기_생성(String uri, String email, String apiToken) {

        return WebClient.builder()
                .baseUrl(uri)
                .defaultHeader(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_VALUE)
                .defaultHeader(HttpHeaders.ACCEPT, MediaType.APPLICATION_JSON_VALUE)
                .defaultHeader("Authorization", "Basic " + getBase64Credentials(email, apiToken))
                .build();
    }

    public JiraRestClient 온프레미스_통신기_생성(String jiraUrl, String jiraID, String jiraPass) {

        final AsynchronousJiraRestClientFactory factory = new AsynchronousJiraRestClientFactory();

        try {
            return factory.createWithBasicHttpAuthentication(new URI(jiraUrl), jiraID, jiraPass);
        }
        catch (URISyntaxException e) {
            String errorMessage = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(),
                    String.format("%s :: [%s] :: 온프레미스_통신기_생성 중 오류", jiraUrl, jiraID));
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private String getBase64Credentials(String jiraID, String jiraPass) {
        String credentials = jiraID + ":" + jiraPass;
        return new String(Base64.getEncoder().encode(credentials.getBytes()));
    }

    public <T> Mono<T> get(WebClient webClient, String uri, Class<T> responseType) {

        return webClient.get()
                .uri(uri)
                .retrieve()
                .bodyToMono(responseType);
    }

    public <T> Mono<T> get(WebClient webClient, String uri, ParameterizedTypeReference<T> elementTypeRef) {

        return webClient.get()
                .uri(uri)
                .retrieve()
                .bodyToMono(elementTypeRef);
    }
    public <T> Mono<T> getWithRawData(WebClient webClient, String uri, Class<T> responseType) {
        return webClient.get()
                .uri(uri)
                .retrieve()
                .bodyToMono(String.class)
                .flatMap(rawData -> {
                    try {
                        ObjectMapper objectMapper = new ObjectMapper();
                        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);

                        T mappedData = objectMapper.readValue(rawData, responseType);

                        if (mappedData instanceof 지라이슈_데이터) {
                            ((지라이슈_데이터) mappedData).setRawData(rawData);
                        } else if (mappedData instanceof 지라이슈조회_데이터) {
                            지라이슈조회_데이터 조회_데이터 = (지라이슈조회_데이터) mappedData;

                            JsonNode rootNode = objectMapper.readTree(rawData);
                            JsonNode issuesNode = rootNode.get("issues"); // 원본 데이터에서 issues 필드 추출

                            if (issuesNode != null && 조회_데이터.getIssues() != null) {
                                for (int i = 0; i < 조회_데이터.getIssues().size(); i++) {
                                    지라이슈_데이터 issue = 조회_데이터.getIssues().get(i);

                                    // 각각의 이슈에 해당하는 원본 JSON 데이터를 추출하여 rowData로 설정
                                    String issueRawData = issuesNode.get(i).toString();
                                    issue.setRawData(issueRawData);
                                }
                            }
                        }

                        return Mono.just(mappedData);
                    } catch (Exception e) {
                        log.error("ROW DATA JSON 매핑 중 오류 발생: " + e.getMessage());
                        return Mono.error(new RuntimeException("ROW DATA JSON 매핑 중 오류 발생", e));
                    }
                });
    }


    public <T> Mono<T> post(WebClient webClient, String uri, Object requestBody, Class<T> responseType) {

        return webClient.post()
                .uri(uri)
                .body(BodyInserters.fromValue(requestBody))
                .retrieve()
                .bodyToMono(responseType);
    }

    public <T> Mono<T> put(WebClient webClient, String uri, Object requestBody, Class<T> responseType) {

        return webClient.put()
                .uri(uri)
                .body(BodyInserters.fromValue(requestBody))
                .retrieve()
                .bodyToMono(responseType);
    }

    public <T> Mono<T> delete(WebClient webClient, String uri, Class<T> responseType) {

        return webClient.delete()
                .uri(uri)
                .retrieve()
                .bodyToMono(responseType);
    }

    public CommonResponse.ApiResult<?> executePost(WebClient webClient, String uri, Object requestBody) {

        Mono<ResponseEntity<Void>> response = webClient.post()
                .uri(uri)
                .body(BodyInserters.fromValue(requestBody))
                .retrieve()
                .toEntity(Void.class);

        return response.map(entity -> {
                    if (entity.getStatusCode() == HttpStatus.NO_CONTENT) {
                        return CommonResponse.success("OK");
                    } else {
                        return CommonResponse.error("Fail", HttpStatus.BAD_REQUEST);
                    }
                })
                .onErrorResume(WebClientResponseException.class, e -> Mono.just(CommonResponse.error(e, e.getStatusCode())))
                .onErrorResume(Exception.class, e -> Mono.just(CommonResponse.error(e, HttpStatus.BAD_REQUEST)))
                .block();
    }

    public CommonResponse.ApiResult<?> executePut(WebClient webClient, String uri, Object requestBody) {

        Mono<ResponseEntity<Void>> response = webClient.put()
                .uri(uri)
                .body(BodyInserters.fromValue(requestBody))
                .retrieve()
                .toEntity(Void.class);

        return response.map(entity -> {
                    if (entity.getStatusCode() == HttpStatus.NO_CONTENT) {
                        return CommonResponse.success("OK");
                    } else {
                        return CommonResponse.error("Fail", HttpStatus.BAD_REQUEST);
                    }
                })
                .onErrorResume(WebClientResponseException.class, e -> Mono.just(CommonResponse.error(e, e.getStatusCode())))
                .onErrorResume(Exception.class, e -> Mono.just(CommonResponse.error(e, HttpStatus.BAD_REQUEST)))
                .block();
    }

    public CommonResponse.ApiResult<?> executeDelete(WebClient webClient, String uri) {

        Mono<ResponseEntity<Void>> response = webClient.delete()
                .uri(uri)
                .retrieve()
                .toEntity(Void.class);

        return response.map(entity -> {
                    if (entity.getStatusCode() == HttpStatus.NO_CONTENT) {
                        return CommonResponse.success("OK");
                    } else {
                        return CommonResponse.error("Fail", HttpStatus.BAD_REQUEST);
                    }
                })
                .onErrorResume(WebClientResponseException.class, e -> Mono.just(CommonResponse.error(e, e.getStatusCode())))
                .onErrorResume(Exception.class, e -> Mono.just(CommonResponse.error(e, HttpStatus.BAD_REQUEST)))
                .block();
    }

    public static LocalDateTime roundToNearest30Minutes(LocalDateTime dateTime) {
        long minutes = dateTime.getMinute();
        int remainder = (int) (minutes % 30);
        return dateTime.plusMinutes(remainder >= 30 ? 30 - remainder : -remainder);//30분 단위 리턴 시간
    }

    public Map<String, CloudJiraIssueCreationFieldMetadata.FieldMetadata> checkFieldMetadata(WebClient webClient, String 프로젝트_아이디, String 이슈유형_아이디) {
        String 필드확인endpoint = 지라API_정보.프로젝트키_대체하기(
                    지라API_정보.getEndpoint().getIssue().getCreatemeta(), 프로젝트_아이디);
        필드확인endpoint = 지라API_정보.이슈유형키_대체하기(필드확인endpoint, 이슈유형_아이디);

        int 검색_시작_지점 = 0;
        int 최대_검색수 = 지라API_정보.getParameter().getMaxResults();
        boolean isLast = false;

        List<CloudJiraIssueCreationFieldMetadata.FieldMetadata> 메타데이터_목록 = new ArrayList<>(); // 이슈 저장

        CloudJiraIssueCreationFieldMetadata cloudJiraIssueCreationFieldMetadata;
        try {
            while (!isLast) {
                String endpoint = 필드확인endpoint +
                        "?startAt=" + 검색_시작_지점 + "&maxResults=" + 최대_검색수;

                cloudJiraIssueCreationFieldMetadata
                        = this.get(webClient, endpoint, CloudJiraIssueCreationFieldMetadata.class).block();

                if (cloudJiraIssueCreationFieldMetadata == null) {
                    log.info("클라우드 지라 클라우드 프로젝트 : {}, 이슈유형 : {}, 이슈생성필드_메타데이터 목록이 없습니다."
                            , 프로젝트_아이디, 이슈유형_아이디);
                    return null;
                }
                else if (cloudJiraIssueCreationFieldMetadata.getFields() == null || cloudJiraIssueCreationFieldMetadata.getFields().size() == 0) {
                    log.info("클라우드 지라 클라우드 프로젝트 : {}, 이슈유형 : {}, " +
                        "이슈생성필드_메타데이터 목록이 없습니다.", 프로젝트_아이디, 이슈유형_아이디);
                    return null;
                }

                메타데이터_목록.addAll(cloudJiraIssueCreationFieldMetadata.getFields());

                if (cloudJiraIssueCreationFieldMetadata.getTotal() == 메타데이터_목록.size()) {
                    isLast = true;
                } else {
                    검색_시작_지점 += 최대_검색수;
                }
            }
        }
        catch (Exception e) {
            log.error("클라우드 지라 프로젝트 : {}, 이슈유형 : {}, 이슈생성필드_메타데이터 확인하기 중 오류"
                    , 프로젝트_아이디, 이슈유형_아이디);
            String 에러로그 = ErrorLogUtil.exceptionLoggingAndReturn(e, this.getClass().getName(), "필드_메타데이터 확인하기");
            throw new IllegalArgumentException(에러로그 + "\n필드 메타데이터 조회 중 오류 :: 프로젝트 :: " + 프로젝트_아이디 +
                                                    " :: 이슈 유형 :: " + 이슈유형_아이디 + " :: 에러 메세지 :: " + 에러로그);
        }

        Map<String, CloudJiraIssueCreationFieldMetadata.FieldMetadata> 필드맵 = 메타데이터_목록.stream()
                .collect(Collectors.toMap(CloudJiraIssueCreationFieldMetadata.FieldMetadata::getFieldId, field -> field));

        return 필드맵;
    }

    public static boolean 전일_업데이트여부(String dateTimeStr) {


        log.info("전일_업데이트여부 실행");

        // 가능한 날짜와 시간 형식 목록
        String[] possibleFormats = {
            "yyyy-MM-dd'T'HH:mm:ss.SSSZ",
            "yyyy-MM-dd'T'HH:mm:ssZ",
            "yyyy-MM-dd'T'HH:mm:ss.SSSXXX",
            "yyyy-MM-dd'T'HH:mm:ssXXX",
        };

        ZonedDateTime inputDateTime = null;

        for (String format : possibleFormats) {
            try {
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(format);
                inputDateTime = ZonedDateTime.parse(dateTimeStr, formatter);
                break;
            } catch (DateTimeParseException e) {
                // 날짜 형식이 일치하지 않을 시 다음 형식으로 시도
            }
        }

        if (inputDateTime == null) {
            log.error("해당 날짜포맷은 지원하지 않는 포맷입니다.: " + dateTimeStr);
            return false;
        }

        Environment environment = getBean(Environment.class);

        Boolean isDevProfile = Arrays.stream(environment.getActiveProfiles())
                .findFirst()
                .map(a -> ApplicationProfile.valueOf(a.toUpperCase()).equals(DEV))
                .orElse(false);


        if(isDevProfile){
            DevDurationConfig devDurationConfig = getBean(DevDurationConfig.class);
            log.info("지금부터_24시간_이전_업데이트가_있다면 실행");
            ZonedDateTime startOfYesterday = ZonedDateTime.now().minusHours(devDurationConfig.getDiscoveryDurationTime());
            ZonedDateTime endOfYesterday = ZonedDateTime.now();
            return inputDateTime.isAfter(startOfYesterday) && inputDateTime.isBefore(endOfYesterday);
        }else{
            ZonedDateTime startOfYesterday = ZonedDateTime.now().minusDays(1).withHour(0).withMinute(0).withSecond(0).withNano(0);
            ZonedDateTime endOfYesterday = startOfYesterday.withHour(23).withMinute(59).withSecond(59);
            return inputDateTime.isAfter(startOfYesterday) && inputDateTime.isBefore(endOfYesterday);
        }

    }

    public static LocalDate getNextDate(String dateString) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        LocalDate date = LocalDate.parse(dateString, formatter);

        return date.plusDays(1);
    }

    public static LocalDate toLocalDate(String dateString) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");

        LocalDate date = LocalDate.parse(dateString, formatter);

        return date;
    }

    public static boolean isDateInRange(LocalDate currentDate,LocalDate startDate,LocalDate endDate) {
        if (!currentDate.isBefore(startDate) && !currentDate.isAfter(endDate)) {
            return true;
        }
        return false;
    }


}
